@@ -246,11 +246,11 @@ impl Einsum {
         }
     }
 
-    pub fn scalar_mul(node: &CircuitRc, scalar: f64, name: Option<String>) -> Self {
+    pub fn scalar_mul(node: CircuitRc, scalar: f64, name: Option<String>) -> Self {
         let axes: EinsumAxes = (0..node.info().rank()).map(|x| x as u8).collect();
         Einsum::try_new(
             vec![
-                (node.clone(), axes.clone()),
+                (node, axes.clone()),
                 (ScalarConstant::new(scalar, sv![], None).rc(), sv![]),
             ],
             axes,
@@ -259,6 +259,15 @@ impl Einsum {
         .unwrap()
     }
 
+    pub fn empty(name: Option<String>) -> Self {
+        Einsum::try_new(vec![], sv![], name).unwrap()
+    }
+
+    pub fn identity(node: CircuitRc, name: Option<String>) -> Self {
+        let axes: EinsumAxes = (0..node.info().rank()).map(|x| x as u8).collect();
+        Einsum::try_new(vec![(node, axes.clone())], axes, name).unwrap()
+    }
+
     pub fn axes_in_input(&self) -> HashSet<EinInt> {
         // could be 256 bit bitmask
         self.input_axes().flatten().copied().collect()
