@@ -29,6 +29,7 @@ pub struct PyUtils {
     einsum: PyObject,
     make_diagonal: PyObject,
     make_bytes: PyObject,
+    pub print: PyObject,
 }
 
 /// misc python utilities
@@ -66,6 +67,7 @@ pub static PY_UTILS: GILLazyPy<PyUtils> = GILLazyPy::new_py(|py| {
         einsum: get("einsum"),
         make_diagonal: get("make_diagonal"),
         make_bytes: get("make_bytes"),
+        print: get("print"),
     }
 });
 
