@@ -540,7 +540,7 @@ impl IntoPy<PyObject> for TensorIndex {
     }
 }
 
-pub fn compose(top: &TensorIndex, bottom: &TensorIndex) -> TensorIndex {
+pub fn compose(top: &TensorIndex, bottom: &TensorIndex) -> Option<TensorIndex> {
     let mut result: Vec<TensorAxisIndex> = vec![];
     let mut top_idx: usize = 0;
     for bottom_idx in bottom.0.iter() {
@@ -550,6 +550,9 @@ pub fn compose(top: &TensorIndex, bottom: &TensorIndex) -> TensorIndex {
                 let top_here = top.0[top_idx].clone();
                 top_idx += 1;
                 Python::with_gil(|py| {
+                    dbg!(top, bottom);
+                    let z: Option<bool> = pycall!(PY_UTILS.print, (top.clone(),));
+                    let z: Option<bool> = pycall!(PY_UTILS.print, (bottom.clone(),));
                     let indexed_tensor = t.clone().py_getitem(py, top_here).unwrap();
                     if indexed_tensor.shape().is_empty() {
                         result.push(TensorAxisIndex::Single(pycall!(
@@ -566,10 +569,22 @@ pub fn compose(top: &TensorIndex, bottom: &TensorIndex) -> TensorIndex {
                 let stop = slice.stop;
                 let top_here = top.0[top_idx].clone();
                 match top_here {
-                    TensorAxisIndex::Single(i) => result.push(TensorAxisIndex::Single(start + i)),
-                    TensorAxisIndex::Tensor(t) => Python::with_gil(|py| {
-                        result.push(TensorAxisIndex::Tensor(t.py_add(py, start).unwrap()))
-                    }),
+                    TensorAxisIndex::Single(i) => {
+                        if i < 0 {
+                            result.push(TensorAxisIndex::Single(stop.unwrap_or(0) + i))
+                        } else {
+                            result.push(TensorAxisIndex::Single(start + i))
+                        }
+                    }
+                    TensorAxisIndex::Tensor(t) => {
+                        if start < 0 {
+                            return None;
+                        } else {
+                            Python::with_gil(|py| {
+                                result.push(TensorAxisIndex::Tensor(t.py_add(py, start).unwrap()))
+                            })
+                        }
+                    }
                     TensorAxisIndex::Slice(top_slice) => {
                         let top_start = top_slice.start.unwrap_or(0);
                         let top_stop = top_slice.stop;
@@ -585,17 +600,24 @@ pub fn compose(top: &TensorIndex, bottom: &TensorIndex) -> TensorIndex {
                                 new_stop = Some(stop.unwrap_or(0) + top_stop.unwrap_or(0));
                             }
                         }
+                        // if you add negative and positive indices and get 0, you really meant "end", or None
+                        if (top_stop.is_some() && top_stop.unwrap() < 0 || start < 0)
+                            && new_stop == Some(0)
+                        {
+                            new_stop = None
+                        }
                         result.push(TensorAxisIndex::Slice(Slice {
                             start: new_start,
                             stop: new_stop,
                         }))
                     }
                 }
+                // dbg!(top, bottom);
                 top_idx += 1;
             }
         }
     }
-    TensorIndex(result)
+    Some(TensorIndex(result))
 }
 
 pub fn cuda_to_first_cuda(string: &str) -> String {
